
/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        


package org.apache.poi.hslf.model;

import java.util.*;
import java.io.*;

import org.apache.poi.util.LittleEndian;
import org.apache.poi.hslf.exceptions.InvalidRecordFormatException;

/**
 * This class represents as sheet in a PowerPoint Document. It allows 
 *  access to the text within. It is only a very basic class for now
 *
 * @author Nick Burch
 */

public class Sheet
{

  private int _sheetNo;
  private byte[] _sheet;
  private TextRun[] _runs;

  /**
   * Constructs a Sheet from the given section of the underlying
   * powerpoint file. Copies useful things into data structures,
   * and initalises the TextRuns it contains
   *
   * @param bytes the byte array to get data from
   * @param start the point in the byte array to start at
   * @param length how far into the byte array to read
   */
  public Sheet(byte[] bytes, int start, int length)
  {
	_sheet = new byte[length];
	System.arraycopy(bytes,start,_sheet,0,length);

	// Grab the sheet number
	_sheetNo = (int)_sheet[7];
	//System.out.println("Created sheet # " + _sheetNo + ", length is " + length);

	// Find the text runs
	// (Text runs occur no earlier than 28 bytes in)
	_runs = findTextRuns(_sheet,28,length);
  }

  /**
   * Scans through the supplied byte array, looking for 
   * TextRuns. If it finds any, it initalises them.
   *
   * @param b the byte array to get data from
   * @param start the point in the byte array to start at
   * @param length how far into the byte array to read
   */
  protected static TextRun[] findTextRuns(byte[] b, int start, int length) {
	Vector runs = new Vector(10);

	// Look for text runs
	// Really quite a crude way for now
	for(int i=0; i < (length - 10); i++) {
		if(b[i] == 0x00 &&
		   b[i+1] == (0x9f-256) &&
		   b[i+2] == 0x0f &&
		   b[i+3] == 0x04 &&
		   b[i+4] == 0x00 &&
		   b[i+5] == 0x00 &&
		   b[i+6] == 0x00) {
			if((length-i) > 12 ) {
				Integer startsAt = new Integer(i);
				runs.add(startsAt);
			} else {
				// End text run, doesn't contain any entries
			}
		}
	}

	// Chuck on end of our array as last end position
	runs.add(new Integer(length));

	// Loop over TextRun positions, creating
	TextRun[] foundRuns = new TextRun[runs.size()-1];
	for(int i=0; i < (runs.size()-1); i++) {
		int spos = ( (Integer)runs.get(i) ).intValue();
		int epos = ( (Integer)runs.get((i+1)) ).intValue();
		try {
			foundRuns[i] = new TextRun(b,spos,(epos-spos));
		} catch(InvalidRecordFormatException irfe) {
			System.out.println("Text Run between " + spos + " and " + epos + " wasn't valid: " + irfe);
		}
	}

	return foundRuns;
  }


  // Accesser methods follow

  /**
   * Returns an array of all the TextRuns found
   */
  public TextRun[] getTextRuns() { return _runs; }

  /**
   * Returns the sheet number
   */
  public int getSheetNumber() { return _sheetNo; }
} 
