
/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        


package org.apache.poi.hslf.model;

import java.util.*;
import java.io.*;

import org.apache.poi.util.LittleEndian;

import org.apache.poi.hslf.*;
import org.apache.poi.hslf.record.*;

/**
 * This class is a friendly wrapper on top of the more scary HSLFSlideShow.
 *
 * TODO:
 *  - figure out where the sheets and metasheets live from information
 *    in the file, rather than by scanning through the data
 *  - correctly figure out what character encoding is used for non unicode
 *    text runs, rather than relying on the system default
 *  - figure out how to match notes to their correct sheet
 *    (will involve understanding DocSlideList and DocNotesList)
 * 
 * @author Nick Burch
 */

public class SlideShow
{
  // What we're based on
  private HSLFSlideShow _hslfSlideShow;

  // Low level contents, as taken from HSLFSlideShow
  private Record[] _records;

  // Friendly objects for people to deal with
  private Sheet[] _sheets;
  private MetaSheet[] _msheets;

  /**
   *  right now this function takes one parameter: a ppt file, and outputs
   *  the text it can find for it
   */
  public static void main(String args[]) throws IOException
  {
	HSLFSlideShow basefoo = new HSLFSlideShow(args[0]);
	SlideShow foo = new SlideShow(basefoo);

	Sheet[] sheets = foo.getSheets();
	for(int i=0; i<sheets.length; i++) {
		Sheet sheet = sheets[i];
		System.out.println("*Sheet " + sheet.getSheetNumber() + ":");
		TextRun[] runs = sheet.getTextRuns();
		for(int j=0; j<runs.length; j++) {
			TextRun run = runs[j];
			System.out.println("  * Text run " + run.getRunType());
			System.out.println("\n" + run.getText() + "\n");
		}
	}
  }

  /**
   * Constructs a Powerpoint document from the underlying 
   * HSLFSlideShow object. Finds the model stuff from this
   *
   * @param hslfSlideShow the HSLFSlideShow to base on
   */
  public SlideShow(HSLFSlideShow hslfSlideShow) throws IOException
  {
	// Get useful things from our base slideshow
    _hslfSlideShow = hslfSlideShow;
	_records = _hslfSlideShow.getRecords();
	byte[] _docstream = _hslfSlideShow.getUnderlyingBytes();


	// For holding the sheets
	Vector sheets = new Vector(10);
	// For holding the meta sheets
	Vector metaSheets = new Vector(10);

	// Look for Records that hold Sheets and MetaSheets
	for(int i=0; i<_records.length; i++) {
		// Look in Notes for PPDrawings
		if(_records[i] instanceof Notes) {
			Record[] children = _records[i].getChildRecords();
			for(int j=0; j<children.length; j++) {
				if(children[j] instanceof PPDrawing) {
					PPDrawing ppd = (PPDrawing)children[j];
					MetaSheet ms = ppd.getMetaSheet();
					if(ms != null) {
						metaSheets.add(ms);
						//System.out.println("Found a meta sheet at " + i + " " + j);
					} else {
						System.err.println("PPDrawing at " + i + ", " + j + " didn't have a meta sheet!");
					}
				}
			}
		}
		// Look in Document -> SlideListWithText
		if(_records[i].getRecordType() == 1000L) {
			Record[] children = _records[i].getChildRecords();
			for(int j=0; j<children.length; j++) {
				if(children[j].getRecordType() == 4080L) {
					SlideListWithText slwt = (SlideListWithText)children[j];
					Sheet[] s = slwt.getSheets();
					//System.out.println("Found " + s.length + " sheets at " + i + " " + j);
					for(int k=0; k<s.length; k++) {
						sheets.add(s[k]);
					}
				}
			}
		}
	}


	// Put the vectors into arrays
	_sheets = new Sheet[sheets.size()];
	_msheets = new MetaSheet[metaSheets.size()];
	for(int i=0; i<_sheets.length; i++) {
		_sheets[i] = (Sheet)sheets.get(i);
	}
	for(int i=0; i<_msheets.length; i++) {
		_msheets[i] = (MetaSheet)metaSheets.get(i);
	}
  }


  /**
   * Writes out the slideshow file the is represented by an instance of
   *  this class
   * @param out The OutputStream to write to.
   *  @throws IOException If there is an unexpected IOException from the passed
   *            in OutputStream
   */
   public void write(OutputStream out) throws IOException {
	_hslfSlideShow.write(out);
   }


  // Accesser methods follow

  /**
   * Returns an array of all the normal Sheets found in the slideshow
   */
  public Sheet[] getSheets() { return _sheets; }

  /**
   * Returns an array of all the meta Sheets found in the slideshow
   */
  public MetaSheet[] getMetaSheets() { return _msheets; }
}
