
/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        

package org.apache.poi.hslf.record;

import org.apache.poi.util.LittleEndian;
import org.apache.poi.hslf.model.Sheet;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Vector;

/**
 * These are tricky beasts. They contain the text of potentially
 *  many (normal) slides. Something in the Slide record must refer
 *  back to them, which is oh so much fun
 * For now, we scan through looking for interesting bits, then creating
 *  the helpful Sheet from model for them
 *
 * @author Nick Burch
 */

public class SlideListWithText extends Record
{
	private byte[] _contents;
	private long _type;

	private Sheet[] sheets;

	/** 
	 * Create a new holder for slide records
	 */
	protected SlideListWithText(byte[] source, int start, int len) {
		_contents = new byte[len];
		System.arraycopy(source,start,_contents,0,len);
		_type = LittleEndian.getUShort(_contents,2);

		// Find our children
		findSheets();
	}

	private void findSheets() {
		// ** At the moment, still scanning for sheets **
		Vector sheetV = new Vector(5);

		// So we can tell is we spotted one that hasn't been created
		Integer lastAt = null;

		// Loop over the data of this PPDrawing, looking for a 
		// sheet or a meta sheets
		for(int i=0; i < (_contents.length - 28); i++) {
			// Look for new pages
			// These look like:
			//     00 f3 03 14 00 00 00 ##
			// (where ## is the page number, as a byte)
			if(_contents[i] == 0x00 &&
				_contents[i+1] == (0xf3-256) &&
				_contents[i+2] == 0x03 &&
				_contents[i+3] == 0x14 &&
				_contents[i+4] == 0x00 &&
				_contents[i+5] == 0x00 &&
				_contents[i+6] == 0x00 
			) {
				// Sheets seem to contain:
				// 28 bytes from i:
				//   00 9f 0f 04 00 00 00 ## 00 00 00 00 00 
				//   a0 0f (len len) 00 00 (text text text)
				// some crap
				//   00 9f 0f 04 00 00 00 ## 00 00 00 00 00
				//   a0 0f (len len) 00 00 (more text text)
				//
				// First ## is 00 (page title), rest can be any number

				// Have yet to spot what defines if unicode text
				//  (xx 00 xx 00) or normal text (xx xx xx)

				//System.out.println("Found new sheet at " + i);

				// If existing one, create
				if(lastAt != null) {
					int lastAtI = lastAt.intValue();
					Sheet s = new Sheet(_contents,lastAtI, i-lastAtI);
					sheetV.add(s);
				}

				lastAt = new Integer(i);
			}
		}
		if(lastAt != null) {
			int lastAtI = lastAt.intValue();
			Sheet s = new Sheet(_contents,lastAtI,_contents.length-lastAtI);
			sheetV.add(s);
		}

		sheets = new Sheet[sheetV.size()];
		for(int i=0; i<sheetV.size(); i++) {
			sheets[i] = (Sheet)sheetV.get(i);
		}
	}

	/**
	 * Get access to the underlying Sheets
	 */
	public Sheet[] getSheets() { return sheets; }

	/** 
	 * For now, pretend to be an atom
	 */
	public boolean isAnAtom() { return true; }

	/**
	 * Return the value we were given at creation
	 */
	public long getRecordType() { return _type; }

	/** 
	 * We're pretending to be an atom, so return null
	 */
	public Record[] getChildRecords() { return null; }

	/**
	 * Write the contents of the record back, so it can be written
	 *  to disk
	 */
	public void writeOut(OutputStream out) throws IOException {
		out.write(_contents);
	}
}
