
/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        

package org.apache.poi.hslf.util;

import java.io.UnsupportedEncodingException;

/**
 * This class provides ways to turn Strings to bytes and back again, be
 *  they unicode or not in the file
 * (PPT unicode is byte swapped)
 *
 * @author Nick Burch
 */

public class TextMunger
{
  /**
   * Constructs a String from a unicode based length of bytes. Java
   *  assumes the format for UTF-16 is (high)(low), but PPT stores it
   *  as (low)(high), yey...
   *
   * @param b the byte array to get data from
   * @param start the point in the byte array to start at
   * @param len how far into the byte array to read
   */
  public static String pptUnicodeMakeString(byte[] b, int start, int len) throws UnsupportedEncodingException {
	byte[] reversed = new byte[len];
	for(int i=0; i<len; i+= 2) {
		reversed[i] = b[i+start+1];
		reversed[i+1] = b[i+start];
	}
	return new String(reversed,"UTF-16");
  }

  /**
   * Constructs a PPT ordered unicode byte array of a given string
   * @param s String to convert
   */
  public static byte[] pptUnicodeMakeBytes(String s) throws UnsupportedEncodingException {
	byte[] bytes = s.getBytes("UTF-16");
	for(int i=0; i<bytes.length; i+=2) {
		byte lower = bytes[i];
		bytes[i] = bytes[i+1];
		bytes[i+1] = lower;
	}

	// First two bytes are junk
	if(bytes.length <= 2) { return new byte[0]; }
	byte[] ret = new byte[bytes.length-2];
	System.arraycopy(bytes,2,ret,0,ret.length);

    return ret;
  }

  /**
   * For a given unicode string, try to make a us-ascii like version
   * Who knows how PPT handles this internally, so we fake it badly
   * @param s String to convert
   */
  public static byte[] pptFakeUSAscii(String s) throws UnsupportedEncodingException {
	byte[] ub = s.getBytes("UTF-16");
	byte[] b = new byte[ub.length/2];

	for(int i=2; i<ub.length; i+=2) {
		byte lower = ub[i];
		int j = i/2;

		if(lower != 0) {
			// Oh dear. Use "?" which is 63
			b[j] = 63;
		} else {
			// Use higher byte
			b[j] = ub[i+1];
		}
	}

	// First two unicode bytes are junk
	if(b.length <= 1) { return new byte[0]; }
	byte[] ret = new byte[b.length-1];
	System.arraycopy(b,1,ret,0,ret.length);

	return ret;
  }
}
